



clang-14 --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %LIBRARY_BIN%\\clang.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %LIBRARY_BIN%\\clang-cl.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %LIBRARY_BIN%\\clang-cpp.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\clang\\14.0.0\\include\\stdatomic.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
